#include <fstream>
#include <iostream>

#define L_ONLY " < "
#define R_ONLY " > "
#define MATCH " = "
#define UNMATCH " | "
#define MODE_ALL -1
#define MODE_L_ONLY 0
#define MODE_R_ONLY 1
#define MODE_MATCH 2
#define MODE_UNMATCH 3

using namespace std;

int compare(int key_num, int key_start[], int key_length[], const char* input_file_l, const char* input_file_r, int mode);
int compare_key(int key_num, string key_l[], string key_r[]);
void out_l_only(string str, int mode);
void out_r_only(string str, int mode);
void out_match(string str_l, string str_r, int mode);
void out_unmatch(string str_l, string str_r, int mode);

int main(int argc, const char* argv[]) {
	if (argc < 5) {
		cout << endl;
		cout << "Usage: key_diff (key_start key_length)+ input_file_l input_file_r [mode]" << endl;
		cout << endl;
		cout << "[mode]" << endl;
		cout << "0:'" << L_ONLY << "' -> ݂̂ɑ݂s" << endl;
		cout << "1:'" << R_ONLY << "' -> Ê݂ɑ݂s" << endl;
		cout << "2:'" << MATCH << "' -> ق̂Ȃs" << endl;
		cout << "3:'" << UNMATCH << "' -> ق̂s" << endl;
		cout << endl;
		cout << "ex.: key_diff 0 3 4 2 l.txt r.txt 3" << endl;
		cout << "l.txtr.txt0Ԗڂ̕3L[A" << endl;
		cout << "4Ԗڂ̕2L[ƂĔrAق̂s؂o" << endl;
		cout << endl;
		cout << "" << endl;
		cout << "[mode]w肷ƔrʂɑΉs؂o" << endl;
		cout << "[mode]w肵ȂƑSĂ̔rʂo͂" << endl;
		cout << "input_file_lEinput_file_r͋ɃL[ɊւďɐςƂ" << endl;
		cout << "L[ɏd͖̂Ƃ" << endl;
		cout << "L[̊Jnʒu̎w0n܂Ƃ" << endl;
		cout << "L[͌Œ蒷Ƃ" << endl;
		return 1;
	}
	int key_num = (argc - 3) / 2;
	int key_start[key_num];
	int key_length[key_num];
	for (int i = 0; i < key_num; i++) {
		key_start[i] = atoi(argv[(i * 2) + 1]);
		key_length[i] = atoi(argv[(i * 2) + 2]);
	}
	
	if (argc % 2 != 0) {
		return compare(key_num, key_start, key_length, argv[argc - 2], argv[argc - 1], MODE_ALL);
	} else {
		return compare(key_num, key_start, key_length, argv[argc - 3], argv[argc - 2], atoi(argv[argc - 1]));
	}
}

int compare(int key_num, int key_start[], int key_length[], const char* input_file_l, const char* input_file_r, int mode) {
	ifstream ifs_l(input_file_l);
	ifstream ifs_r(input_file_r);
	string buf_l, buf_r, key_l[key_num], key_r[key_num];
	bool read_l = false;
	bool read_r = false;
	int cmp_key_rslt;
	
	while (ifs_l && ifs_r) {
		if (!read_l) {
			read_l = getline(ifs_l, buf_l);
		}
		if (!read_r) {
			read_r = getline(ifs_r, buf_r);
		}
		if (read_l && !read_r) {
			while (read_l) {
				out_l_only(buf_l, mode);
				read_l = getline(ifs_l, buf_l);
			}
			break;
		} else if (!read_l && read_r) {
			while (read_r) {
				out_r_only(buf_r, mode);
				read_r = getline(ifs_r, buf_r);
			}
			break;
		} else if (!read_l && !read_r) {
			break;
		} else {
			for (int i = 0; i < key_num; i++) {
				key_l[i] = buf_l.substr(key_start[i], key_length[i]);
				key_r[i] = buf_r.substr(key_start[i], key_length[i]);
			}
			cmp_key_rslt = compare_key(key_num, key_l, key_r);
			
			if (cmp_key_rslt < 0) {
				out_l_only(buf_l, mode);
				read_l = false;
			} else if (cmp_key_rslt > 0) {
				out_r_only(buf_r, mode);
				read_r = false;
			} else {
				if (buf_l != buf_r) {
					out_unmatch(buf_l, buf_r, mode);
				} else {
					out_match(buf_l, buf_r, mode);
				}
				read_l = false;
				read_r = false;
			}
		}
	}
	
	return 0;
}

int compare_key(int key_num, string key_l[], string key_r[]) {
	for (int i = 0; i < key_num; i++) {
		if (key_l[i] < key_r[i]) {
			return -1;
		} else if (key_l[i] > key_r[i]) {
			return 1;
		}
	}
	return 0;
}

void out_l_only(string str, int mode) {
	if (mode == MODE_ALL) {
		cout << str << L_ONLY << endl;
	} else if (mode == MODE_L_ONLY) {
		cout << str << endl;
	}
}

void out_r_only(string str, int mode) {
	if (mode == MODE_ALL) {
		cout << R_ONLY << str << endl;
	} else if (mode == MODE_R_ONLY) {
		cout << str << endl;
	}
}

void out_match(string str_l, string str_r, int mode) {
	if (mode == MODE_ALL) {
		cout << str_l << MATCH << str_r << endl;
	} else if (mode == MODE_MATCH) {
		cout << str_l << endl;
	}
}

void out_unmatch(string str_l, string str_r, int mode) {
	if (mode == MODE_ALL || mode == MODE_UNMATCH) {
		cout << str_l << UNMATCH << str_r << endl;
	}
}
